/*************************************************
*            BK Emulator for Window 3.0          *
*************************************************/

/*
File: PanelSplitter.cpp
Version: 1.0.0
Written by: Yuriy Kalmykov <kalmykov@stoik.com>
    Copyright (c) 2002-2004 Yuriy Kalmykov

    BK Emulator is a program emulated hardware environment for running
code for BK 0010(01) in different configurations. 
           
    This code may be used in compiled form in any way you desire.
This file or it's parts can't be redistributed without the authors
written consent, but can be modified for your private needs.
    Providing that this notice and the authors name and all copyright
notices remains intact.

    Please, an email me to know how you are using it and where. You can
ask me for any information about this below code or any attendant
knowledge.
    
    This file is provided "as is" with no expressed or implied warranty.
The author accepts no liability for any damage or loss of business that
this product may cause.
*/




// PanelSplitter.cpp: implementation of the CPanelSplitter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PanelSplitter.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPanelSplitter::CPanelSplitter (CWnd* pParent) : CPanel (pParent)
{
	m_splitter.Create (NULL, NULL, WS_VISIBLE|WS_CHILD|WS_CLIPSIBLINGS, CRect (0, 0, 0, 0), pParent, 0);
	m_splitter.PutOnPanel (this);
	m_splitter.SetMouseTransparence (FALSE);
	
	m_bSizeRelativeToWindow = TRUE;

	SetName (NULL);

	m_original_cx = 0.0;
	m_original_cy = 0.0;
	m_fixedWidth = 0;
	m_fixedHeight = 0;
	m_bFixedWidth = FALSE;
	m_bFixedHeight = FALSE;
	m_bParentWidth = FALSE;
	m_bParentHeight = FALSE;

	InsertWindow (&m_splitter, WINDOW_STRETCHED);
}


CPanelSplitter::~CPanelSplitter ()
{
	m_splitter.DestroyWindow ();
	m_splitter.m_hWnd = NULL;
}



//////////////////////////////////////////////////////////////////////
void CPanelSplitter::OnSetAlign (int align)
{
	switch (align)
	{
	case ALIGN_LEFT:
	case ALIGN_RIGHT:
		m_bFixedWidth = TRUE;
		m_bParentHeight = TRUE;
		m_fixedWidth = DEFAULT_SPLITTER_WIDTH;
		break;
	case ALIGN_TOP:
	case ALIGN_BOTTOM:
		m_bFixedHeight = TRUE;
		m_bParentWidth = TRUE;
		m_fixedHeight = DEFAULT_SPLITTER_WIDTH;
		break;
	default:
		ASSERT (FALSE);
	}
}



//////////////////////////////////////////////////////////////////////
void CPanelSplitter::OnDraw (CDC* pDC, CRect& rcBound)
{
	// Call of parent panel virtual draw function
	GetParent ()->OnDrawSplitter (pDC, rcBound);
}



//////////////////////////////////////////////////////////////////////
void CPanelSplitter::OnRecalcWindowsZorder ()
{
	::BringWindowToTop (m_hWnd);
	m_splitter.BringWindowToTop ();

	CPanel::OnRecalcWindowsZorder ();
}